from .paginator import *
from .objects import *

__title__ = 'pycord-paging'
__summary__ = 'Pycord paginator for messages and embeds.'
__uri__ = 'https://github.com/Tommodev-06/pycord-paging'
__author__ = 'Tommodev'
__version__ = '1.0.0'
__license__ = 'MIT'
__copyright__ = '2022-present Tommodev.'
